package com.zh.project0512.controller.wxApp;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.AppMessage;
import com.zh.project0512.model.User;
import com.zh.project0512.service.IAppMessageService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

/**
 * <p>
 * app消息
 * </p>
 *
 * @author zh
 * @since 2022-05-26
 */
@RestController
@RequestMapping("/wxApp/userMessage")
@Tag(name = "用户消息")
public class AppMessageController {
    @Autowired
    private IAppMessageService appMessageService;

    @Operation(summary = "消息数量")
    @PostMapping("/count")
    @TokenValid
    public Result total( @RequestHeader(value = "token") @Parameter(name = "登录token") String token) {
        return Result.success(appMessageService.countUnsettledMsg(new JwtUtil().parseOpenid(token)));
    }
    @Operation(summary = "消息列表")
    @PostMapping("/list")
    @TokenValid
    public Result list(@RequestBody JSONObject obj, @RequestHeader(value = "token") @Parameter(name = "登录token") String token) {
        String openid = new JwtUtil().parseOpenid(token);
        User user = userService.selByOpenid(openid);
        QueryWrapper<AppMessage> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("receiverId", user.getId());
        queryWrapper.orderByAsc("settled");
        queryWrapper.orderByDesc("createDate");
        return Result.success(appMessageService.page(MybatisPlusUtil.SetPage(obj), queryWrapper), "请求成功");
    }

    @Autowired
    private IUserService userService;

    @Data
    static class AMParam {
        @NotNull(message = "消息id不能为空")
        private Integer id;
    }

    @Operation(summary = "标记已读")
    @PostMapping("/read")
    @TokenValid
    public Result read(@RequestBody AMParam param, @RequestHeader(value = "token") @Parameter(name = "登录token") String token) {
        UpdateWrapper<AppMessage> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", param.getId()).set("settled", 1);
        return MybatisPlusUtil.sqlResult(appMessageService.update(updateWrapper), "标记");
    }
    @Operation(summary = "全部已读")
    @PostMapping("/readAll")
    @TokenValid
    public Result readAll(@RequestHeader(value = "token") @Parameter(name = "登录token") String token) {
        Claims claims = new JwtUtil().parseJWT(token);
        Integer useId = userService.selByOpenid(claims.getId()).getId();
        UpdateWrapper<AppMessage> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("receiverId", useId).eq("settled",0).set("settled", 1);
        return MybatisPlusUtil.sqlResult(appMessageService.update(updateWrapper), "标记");
    }
}
