package com.zh.project0512.controller.wxApp;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.User;
import com.zh.project0512.model.UserPointsRecords;
import com.zh.project0512.service.IUserPointsRecordsService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;

/**
 * 用户积分
 */
@Tag(name = "用户积分")
@RestController
@RequestMapping("/wxApp/userPoints")
public class UserPointsRecordsController {
    @Autowired
    private IUserPointsRecordsService userPointsRecordsService;
    @Autowired
    private IUserService userService;
//    @Resource
//    IUserPointsRecordsService userPointsRecordsService;
//
//

    /**
     * 查询用户积分详情
     */
    @Operation(summary = "用户积分详情")
    @PostMapping("/detail")
    @TokenValid
    public Result pointsDetail(@RequestHeader String token) {
        String openid = new JwtUtil().parseOpenid(token);
        User user = userService.selByOpenid(openid);
        if(user==null){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        return Result.success(new JSONObject().fluentPut("points", user.getPoints()).fluentPut("historyPoints", user.getHistoryPoints()), "请求成功");
    }

    @Data
    static class UPRParam {
        @NotNull(message = "year不能为空")
        @Schema(title = "年份")
        private Integer year;
        @NotNull(message = "month不能为空")
        @Schema(title = "月份")
        private Integer month;
    }

    /**
     * 查询用户积分记录
     */
    @Operation(summary = "用户积分记录")
    @PostMapping("/list")
    @TokenValid
    public Result list(@Validated @RequestBody UPRParam param, @RequestHeader String token) {
        String openid = new JwtUtil().parseOpenid(token);
        User user = userService.selByOpenid(openid);
        QueryWrapper<UserPointsRecords> qw = new QueryWrapper<>();
        qw.eq("userId", user.getId());
        LocalDate now = LocalDate.of(param.getYear(), param.getMonth(),1);
        String format = now.format(DateTimeFormatter.ofPattern("yyyy-MM"));
        qw.eq("date_format(createDate, '%Y-%m')", format);
        qw.orderByDesc("createDate");
        return Result.success(userPointsRecordsService.list(qw), "请求成功");
    }


}

