package com.zh.project0512.controller.wxApp;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.project0512.annotation.TokenValid;
import com.zh.project0512.model.Video;
import com.zh.project0512.service.IUserAccountService;
import com.zh.project0512.service.IUserService;
import com.zh.project0512.service.IUserTaskService;
import com.zh.project0512.service.IVideoService;
import com.zh.project0512.utils.JwtUtil;
import com.zh.project0512.utils.MybatisPlusUtil;
import com.zh.project0512.utils.RegexUtil;
import com.zh.project0512.utils.RegexUtils;
import com.zh.project0512.utils.result.HttpStatusEnum;
import com.zh.project0512.utils.result.Result;
import io.jsonwebtoken.Claims;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-05-18
 */
@RestController
@RequestMapping("/wxApp/video")
@Tag(name = "视频")
public class VideoUController {
    @Autowired
    private IVideoService videoService;
    @Autowired
    private IUserService userService;
    @Autowired
    private IUserTaskService userTaskService;
    @Autowired
    private IUserAccountService userAccountService;

    @Data
    static class UserAddParam {
        @NotNull(message = "任务id不能为空")
        private Integer id;
        @NotNull(message = "type不能为空")
        @Min(value = 1, message = "视频范围为:1抖音；2快手；3朋友圈；4视频号")
        @Max(value = 4, message = "视频范围为:1抖音；2快手；3朋友圈；4视频号")
        @Schema(title = "平台", description = "1抖音；2快手；3朋友圈；4视频号")
        private Integer type;
        @NotNull(message = "视频地址不能为空")
        private String url;
    }

    @Operation(summary = "用户上传")
    @PostMapping("userAdd")
    @TokenValid
    public Result userAdd(@Validated @RequestBody UserAddParam param, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Claims claims = new JwtUtil().parseJWT(token);
        Integer taskId = param.getId();
        Integer useId = userService.selByOpenid(claims.getId()).getId();
//        校验用户和用户账号
        if (useId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        Integer userAccountStatus = userAccountService.userAccountStatus(useId, param.getType());
        if (userAccountStatus == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "请先申请关联账号");
        } else if (userAccountStatus != 1) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "账号不可用");
        }
//        校验任务状态
        Integer taskStatus = userTaskService.userTaskStatus(useId, taskId);
        if (taskStatus == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "用户未接受任务");
        } else if (taskStatus != 1) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "任务不在上传时间内");
        }
        // 不支持重复提交
        QueryWrapper<Video> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", useId).eq("taskId", taskId).eq("type", param.getType()).ne("status",3);
        if (videoService.getMap(queryWrapper) != null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "已提交过该平台视频");
        }
        // 校验并提取链接
        String s = RegexUtils.get("(http|https)://[A-Za-z0-9_\\-\\+.:?&@=/%#,;]*",param.getUrl(),0);
        if(null == s){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "视频链接有误");
        }else if(param.getType() == 1&&0==StringUtils.countOccurrencesOf(s,"https://v.douyin")){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "请确认平台是否选择正确");
        }else if(param.getType() == 2&&0==StringUtils.countOccurrencesOf(s,"https://v.kuaishou")){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "请确认平台是否选择正确");
        }
        Map<String, Object> urlRepeat = videoService.getMap(new QueryWrapper<Video>().eq("url", s).ne("status",3));
        if(null !=urlRepeat){
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "视频链接已被提交过");
        }
        videoService.save(new Video().setUserId(useId).setTaskId(taskId).setType(param.getType()).setUrl(s).setCreatAt(LocalDateTime.now()).setUpdateAt(LocalDateTime.now()));
        return Result.success("添加完成！");
    }

    @Operation(summary = "用户上传记录")
    @PostMapping("addedRecord")
    @TokenValid
    public Result addedRecord(@RequestBody(required = false) JSONObject obj, @RequestHeader("token") @Parameter(name = "登录token") String token) {
        Claims claims = new JwtUtil().parseJWT(token);
        Integer useId = userService.selByOpenid(claims.getId()).getId();
        if (useId == null) {
            return Result.fail(HttpStatusEnum.CUSTOM_EXCEPTION, "未找到该用户");
        }
        QueryWrapper<Video> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", useId);
        Integer taskId = obj.getInteger("taskId");
        if (taskId != null) {
            queryWrapper.eq("taskId", taskId);
        }
        return Result.success(videoService.page(MybatisPlusUtil.SetPage(obj), queryWrapper));
    }
}
