package com.zh.project0512.mapper;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.project0512.model.Reference;
import com.zh.project0512.model.Task;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import org.apache.ibatis.annotations.*;
import org.apache.poi.ss.formula.functions.T;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * Mapper 接口
 * </p>
 *
 * @author zh
 * @since 2022-05-21
 */
public interface TaskMapper extends BaseMapper<Task> {
    @Insert("insert into task (title,subtitle,start,end) values (#{title},#{subtitle},#{start},#{end})")
    int add(Task task);

    @Select("<script>" +
        "SELECT t1.id " +
        "from task t1 " +
        "WHERE t1.id " +

        "<if test='tagIdList !=null and tagIdList.size()!=0'> " +
        "and t1.id in (SELECT taskId from taskTag WHERE taskTag.tagId in " +
        "<foreach collection='tagIdList' index='index' item='item' open='(' separator=',' close=')'>#{item}</foreach> " +
        ")" +
        "</if>" +

        "<if test='brandIdList !=null and brandIdList.size()!=0'> " +
        "and t1.id in (SELECT taskId from taskBrand WHERE taskBrand.brandId in " +
        "<foreach collection='brandIdList' index='index' item='item' open='(' separator=',' close=')'>#{item}</foreach> " +
        ")" +
        "</if>" +

        "<if test='openid!=null'>" +
        "and IF((SELECT count(*)!=0 FROM taskDepartment WHERE taskId = t1.id)>0," +
        "t1.id in " +
//        "(SELECT taskId from taskDepartment inner JOIN user on user.openid =#{openid} " +
//        "and user.main_department = taskDepartment.departmentId), " +
        "(SELECT taskId from taskDepartment WHERE departmentId in "+
        "(SELECT departmentId from qywxDepartmentUserLink WHERE openid = #{openid}) GROUP BY taskId),"+
        " 1=1)" +
        "</if>" +

        "<if test='ew != null'>" +
        "<if test='ew.nonEmptyOfWhere'>" +
        "AND " +
        "</if> " +
        "${ew.SqlSegment}" +
        "</if> " +

        "ORDER BY t1.updateAt DESC,t1.creatAt DESC" +
        "</script>")
    IPage<Integer> idList(IPage iPage, String openid, List<Integer> tagIdList, List<Integer> brandIdList, @Param("ew") Wrapper<Reference> queryWrapper);

    List<Map> list(String openid, List<Integer> list, List<Integer> tagIdList, List<Integer> brandIdList, @Param("ew") Wrapper<Reference> queryWrapper);

    Task detailById(String openid, int id);

    @Select("SELECT count(*) as taskUserSum from user as t1\n" +
        "left join taskDepartment as t2 on t1.main_department = t2.departmentId\n" +
        "WHERE t2.taskId = #{taskId}")
    Integer taskUserSum(Integer taskId);

    /**
     * 统计任务上传视频人数
     * @param taskId
     * @return
     */
    @Select("SELECT count(userId) as taskUserNum from (SELECT userId from task as t1\n" +
        " INNER JOIN video as t2 on t1.id = t2.taskId\n" +
        "WHERE t1.id = #{taskId}\n" +
        "GROUP BY userId) as taskUser")
    Integer taskUserNum(Integer taskId);

    @Select("SELECT IFNULL(SUM(t1.playNum),0) as playNumSum,IFNULL(SUM(t1.commendNum),0) as commendNumSum,\n" +
        "IFNULL(SUM(t1.collectionNum),0) as collectionNumSum,IFNULL(SUM(t1.commentNum),0) as commentNumSum,\n" +
        "IFNULL(SUM(t1.reSendNum),0) as reSendNumSum,IFNULL(SUM(t1.recommendNum),0) as recommendNumSum,\n" +
        "IFNULL(SUM(t1.effectResult),0) as effectResultSum,t1.userId,\n" +
        "t2.title as taskTitle from video as t1 INNER JOIN task as t2 on t2.id = t1.taskId\n" +
        "LEFT JOIN user as t3 on t1.userId = t3.id\n" +
        "LEFT JOIN qywxDepartment as t4 on t3.main_department = t4.departmentId " +
        "${ew.customSqlSegment}")
    JSONObject taskEffect(@Param("ew") Wrapper<Task> queryWrapper);

    @Select("SELECT IFNULL(SUM(t1.playNum),0) as playNumSum,IFNULL(SUM(t1.commendNum),0) as commendNumSum,\n" +
        "IFNULL(SUM(t1.collectionNum),0) as collectionNumSum,IFNULL(SUM(t1.commentNum),0) as commentNumSum,\n" +
        "IFNULL(SUM(t1.reSendNum),0) as reSendNumSum,IFNULL(SUM(t1.recommendNum),0) as recommendNumSum,\n" +
        "IFNULL(SUM(t1.effectResult),0) as effectResultSum,t1.userId,\n" +
        "t2.title as taskTitle,t3.name as userName,t4.name as departmentName from video as t1 INNER JOIN task as t2 on t2.id = t1.taskId\n" +
        "LEFT JOIN user as t3 on t1.userId = t3.id\n" +
        "LEFT JOIN qywxDepartment as t4 on t3.main_department = t4.departmentId " +
        "${ew.customSqlSegment}")
    List<Map> taskEffectData(@Param("ew") Wrapper<Task> queryWrapper);

    @Select("SELECT IFNULL(SUM(t1.playNum),0) as playNumSum,IFNULL(SUM(t1.commendNum),0) as commendNumSum,\n" +
        "IFNULL(SUM(t1.collectionNum),0) as collectionNumSum,IFNULL(SUM(t1.commentNum),0) as commentNumSum,\n" +
        "IFNULL(SUM(t1.reSendNum),0) as reSendNumSum,IFNULL(SUM(t1.recommendNum),0) as recommendNumSum,\n" +
        "IFNULL(SUM(t1.effectResult),0) as effectResultSum,t1.userId,\n" +
        "t2.title as taskTitle,t3.name as userName,t4.name as departmentName from video as t1 INNER JOIN task as t2 on t2.id = t1.taskId\n" +
        "LEFT JOIN user as t3 on t1.userId = t3.id\n" +
        "LEFT JOIN qywxDepartment as t4 on t3.main_department = t4.departmentId " +
        "${ew.customSqlSegment}")
    IPage<Map> taskEffectDataPage(IPage page,@Param("ew") Wrapper<Task> queryWrapper);

    @Select("select t1.departmentId,t2.name as departmentName from taskDepartment t1 " +
        "inner join qywxDepartment t2 on t1.departmentId = t2.departmentId and t1.taskId =#{taskId}")
    List<Map> taskDepartment(int taskId);
}
