package com.zh.project0512.utils.result;


import com.zh.project0512.utils.page.PageInfo;
import lombok.Data;

import java.util.List;


/**
 * @author jinliwei
 * 返回pageInfo分页
 */
@Data
public class ResultPageInfo<T> {
    public ResultPageInfo(boolean success, int code) {
        this.setSuccess(success);
        this.setCode(code);
    }

    public ResultPageInfo(boolean success, int code, PageInfo<T> data) {
        this.setSuccess(success);
        this.setCode(code);
        this.setData(data);
    }

    public ResultPageInfo(boolean success, int code, String msg) {
        this.setSuccess(success);
        this.setCode(code);
        this.setData(data);
    }

    public ResultPageInfo(boolean success, int code, PageInfo<T> data, String msg) {
        this.setSuccess(success);
        this.setCode(code);
        this.setData(data);
        this.setMsg(msg);
    }

    /**
     * 请求是否成功
     * true:成功
     * false：失败
     */
    private boolean success;

    /**
     * 状态码
     * 成功：200
     * 失败：其他
     */
    private int code;

    /**
     * 失败状态码描述
     * 如果成功不返回
     * 失败返回状态码对应的msg消息
     */
    private String msg;

    /**
     * 请求数据的结果
     */
    private PageInfo<T> data;


    public static <T> ResultPageInfo<T> success() {
        return new ResultPageInfo<T>(true, 200);
    }


    public static <T> ResultPageInfo<T> success(PageInfo<T> data) {
        return new ResultPageInfo<T>(true, 200, data);
    }

    public static <T> ResultPageInfo<T> success(PageInfo<T> data, String msg) {
        return new ResultPageInfo<T>(true, 200, data, msg);
    }



    public static <T> ResultPageInfo<T> fail(HttpStatusEnum httpStatusEnum) {
        return new ResultPageInfo<T>(false, httpStatusEnum.code(), httpStatusEnum.reasonPhraseUS());
    }

    public static <T> ResultPageInfo<T> fail(HttpStatusEnum httpStatusEnum, String msg) {
        return new ResultPageInfo<T>(false, httpStatusEnum.code(), msg);
    }

}
